<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\Pix\Util;

class ApiSicoob{
  
    private $clientId;
    private $certificate;
    private $certificatepass;
    private $homologacao;
    private $baseUrl;
    private $baseUrlAccessToken;
  
    /**
     * Define os dados iniciais da classe
     * @param string $baseUrl
     * @param string $clientId
     * @param string $clientSecret
     * @param string $certificate
     */
    public function __construct($clientId, $certificate, $certificatepass, $homologacao = true){
        if($homologacao){
            $baseUrlAccessToken = '';
            $baseUrl = 'https://sandbox.sicoob.com.br/sicoob/sandbox';
        }else{
            $baseUrlAccessToken = 'https://auth.sicoob.com.br/auth/realms/cooperado/protocol/openid-connect/token';
            $baseUrl = 'https://api.sicoob.com.br/pix-pagamentos/v2';
        } 
        
        $this->baseUrlAccessToken = $baseUrlAccessToken;
        $this->baseUrl            = $baseUrl;
        $this->clientId           = $clientId;
        $this->certificate        = $certificate;
        $this->certificatepass    = $certificatepass;
        $this->homologacao        = $homologacao;
    }
  
    /**
     * Método responsável por iniciar o pagamento
     * @param  string $txid
     * @param  array $request
     * @return array
     */
    public function iniciaPagamento($request){
        return $this->send('POST','/pix-pagamentos/v2/pagamentos', $request);
    }
  
    /**
     * Método responsável por confirmar o pagamento
     * @param  string $txid
     * @param  array $request
     * @return array
     */
    public function confirmaPagamento($request){
        return $this->send('POST','/pix-pagamentos/v2/pagamentos/confirmacao', $request);
    }
  
    /**
     * Método responsável por consultar o pagamento
     * @param  string $txid
     * @param  array $request
     * @return array
     */
    public function consultaPagamento($endtoendid, $request){
        return $this->send('GET','/pix-pagamentos/v2/pagamentos/'.$endtoendid, $request);
    }
  
    /**
     * Método responsável por obter o token de acesso
     * @return string
     */
    private function getAccessToken(){
        //ENDPOINT COMPLETO
        $endpoint = $this->baseUrlAccessToken;
        
        if($endpoint){
            //HEADERS
            $headers = [
                'Content-Type: application/x-www-form-urlencoded'
            ];
        
            //CORPO DA REQUISIÇÃO
            $request = 'grant_type=client_credentials&client_id='.$this->clientId.'&scope=pixpagamentos_escrita%20pixpagamentos_webhook%20pixpagamentos_consulta';
            
            //URL CERTIFICADO/KEY
            $url_certificado = $this->certificate;
            
            //PATH CERTIFICADO/KEY
            $path_certificado = sys_get_temp_dir().DIRECTORY_SEPARATOR.$this->clientId.'certificate.crt';
        
            //INCLUI CERTIFICADO NA TMP(.crt)
            file_put_contents(
                $path_certificado,
                file_get_contents($url_certificado)
            );
            
            //CONFIGURAÇÃO DO CURL
            $curl = curl_init();
            curl_setopt_array($curl, array(
              CURLOPT_URL => $endpoint,
              CURLOPT_RETURNTRANSFER => true,
              CURLOPT_ENCODING => '',
              CURLOPT_MAXREDIRS => 10,
              CURLOPT_TIMEOUT => 0,
              CURLOPT_FOLLOWLOCATION => true,
              CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
              CURLOPT_CUSTOMREQUEST => 'POST',
              CURLOPT_POSTFIELDS => $request,
              CURLOPT_HTTPHEADER => $headers,
              CURLOPT_SSLCERT => $path_certificado,
              CURLOPT_SSLCERTPASSWD => $this->certificatepass
            ));
            
            //EXECUTA O CURL
            $response = curl_exec($curl);
            curl_close($curl);
            
            //REMOVE CERTIFICADO/KEY DA TMP
            unlink($path_certificado);
            unlink($path_private_key);
            
            //RESPONSE EM ARRAY
            $accessToken = json_decode($response,true);
        }else{
            $accessToken = '1301865f-c6bc-38f3-9f49-666dbcfc59c3';
        }
        
        //RETORNA O ACCESS TOKEN
        return $accessToken ?? '';
    }
  
  
    /**
     * Método responsável por enviar requisições para o PSP
     * @param  string $method
     * @param  string $resource
     * @param  array  $request
     * @return array
     */
    private function send($method,$resource,$request = []){
        //ENDPOINT
        $endpoint = $this->baseUrl.$resource;
        
        //HEADERS
        $headers = [
            'Content-Type: application/json',
            'Accept: application/json',
            'Authorization: Bearer '.$this->getAccessToken(),
            'client_id: '.$this->clientId
        ];
        
        //URL CERTIFICADO/KEY
        $url_certificado = $this->certificate;
        
        // VERIFICA AMBIENTE
        if(!$this->homologacao){
            //PATH CERTIFICADO
            $path_certificado = sys_get_temp_dir().DIRECTORY_SEPARATOR.$this->clientId.'certificate.crt';
        
            //INCLUI CERTIFICADO NA TMP(.crt)
            file_put_contents(
                $path_certificado,
                file_get_contents($url_certificado)
            );
        }else{
            $path_certificado = '';
        }
        
        //CONFIGURAÇÃO DO CURL
        $curl = curl_init();
        
        curl_setopt_array($curl, array(
            CURLOPT_URL => $endpoint,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST  => $method,
            CURLOPT_HTTPHEADER => $headers
        ));
        
        if($path_certificado){
            curl_setopt($curl,CURLOPT_SSLCERT,$path_certificado);
            curl_setopt($curl,CURLOPT_SSLCERTPASSWD,$this->certificatepass);
        }
        
        switch ($method) {
            case 'POST':
            case 'PUT':
            case 'PATCH':
                curl_setopt($curl,CURLOPT_POSTFIELDS,json_encode($request));
                break;
        }
        //EXECUTA O CURL
        $response = curl_exec($curl);
        curl_close($curl);
        
        //RESPONSE EM ARRAY
        $responseArray = json_decode($response,true);
        
        //RETORNA O ARRAY DA RESPOSTA
        return $responseArray;
    }
}